package net.knarcraft.launchpad.launchpad;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.launchpad.Launchpad;
import net.knarcraft.launchpad.config.LaunchpadMessage;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * A block registered as a launchpad
 */
public class LaunchpadBlock {

    private final @NotNull Block block;
    private double horizontalVelocity;
    private double verticalVelocity;
    private @Nullable BlockFace fixedDirection;
    private final boolean isImplicit;

    /**
     * Instantiates a new launchpad block
     *
     * @param block      <p>The block to register this launchpad block for</p>
     * @param isImplicit <p>Whether the block in question is implicitly a launchpad, or has been manually created</p>
     */
    public LaunchpadBlock(@NotNull Block block, boolean isImplicit) {
        this.block = block;
        this.horizontalVelocity = -1;
        this.verticalVelocity = -1;
        this.fixedDirection = null;
        this.isImplicit = isImplicit;
    }

    /**
     * Instantiates a new launchpad block
     *
     * @param block              <p>The block to register this launchpad block for</p>
     * @param horizontalVelocity <p>The horizontal velocity of the launchpad</p>
     * @param verticalVelocity   <p>The vertical velocity of the launchpad</p>
     * @param fixedDirection     <p>The fixed direction of the launchpad</p>
     * @param isImplicit         <p>Whether the block in question is implicitly a launchpad, or has been manually created</p>
     */
    public LaunchpadBlock(@NotNull Block block, double horizontalVelocity, double verticalVelocity,
                          @Nullable BlockFace fixedDirection, boolean isImplicit) {
        this.block = block;
        setHorizontalVelocity(horizontalVelocity);
        setVerticalVelocity(verticalVelocity);
        setFixedDirection(fixedDirection);
        this.isImplicit = isImplicit;
    }

    /**
     * Gets the block that contains this launchpad
     *
     * @return <p>This launchpad's block</p>
     */
    public @NotNull Block getBlock() {
        return this.block;
    }

    /**
     * Gets the horizontal velocity of this launchpad
     *
     * @return <p>The horizontal velocity of this launchpad</p>
     */
    public double getHorizontalVelocity() {
        if (this.horizontalVelocity >= 0) {
            return this.horizontalVelocity;
        } else {
            return Launchpad.getInstance().getConfiguration().getHorizontalVelocity(this.block.getType());
        }
    }

    /**
     * Gets the vertical velocity of this launchpad
     *
     * @return <p>The vertical velocity of this launchpad</p>
     */
    public double getVerticalVelocity() {
        if (this.verticalVelocity >= 0) {
            return this.verticalVelocity;
        } else {
            return Launchpad.getInstance().getConfiguration().getVerticalVelocity(this.block.getType());
        }
    }

    /**
     * Gets the fixed direction of this launchpad block
     *
     * @return <p>The fixed direction, or null if not fixed</p>
     */
    public @Nullable BlockFace getFixedDirection() {
        return this.fixedDirection;
    }

    /**
     * Sets the horizontal velocity of this launchpad
     *
     * @param horizontalVelocity <p>The horizontal velocity to set</p>
     */
    public void setHorizontalVelocity(double horizontalVelocity) {
        this.horizontalVelocity = horizontalVelocity;
    }

    /**
     * Sets the vertical velocity of this launchpad
     *
     * @param verticalVelocity <p>The vertical velocity to set</p>
     */
    public void setVerticalVelocity(double verticalVelocity) {
        this.verticalVelocity = verticalVelocity;
    }

    /**
     * Sets the fixed direction of this launchpad
     *
     * @param fixedDirection <p>The fixed direction, or null to unset the fixed direction</p>
     */
    public void setFixedDirection(@Nullable BlockFace fixedDirection) {
        // Make sure the fixed direction is in the correct plane by multiplying it by the normal vector to the 
        // North x West plane.
        if (fixedDirection != null && ((fixedDirection.getDirection().dot(BlockFace.WEST.getDirection().crossProduct(
                BlockFace.NORTH.getDirection()))) != 0 || fixedDirection == BlockFace.SELF)) {
            return;
        }
        this.fixedDirection = fixedDirection;
    }

    /**
     * Gets the actual horizontal velocity specified for this block, with no fallback
     *
     * @return <p>The raw horizontal velocity</p>
     */
    public double getHorizontalVelocityRaw() {
        return this.horizontalVelocity;
    }

    /**
     * Gets the actual vertical velocity specified for this block, with no fallback
     *
     * @return <p>The raw vertical velocity</p>
     */
    public double getVerticalVelocityRaw() {
        return this.verticalVelocity;
    }

    @Override
    public String toString() {
        StringFormatter stringFormatter = Launchpad.getInstance().getStringFormatter();
        String inheritedSuffix = stringFormatter.getUnformattedColoredMessage(
                LaunchpadMessage.SUCCESS_LAUNCHPAD_INFO_INHERITED);

        String directionString = fixedDirection != null ? fixedDirection.name() : "None";
        Location location = block.getLocation();
        String coordinates = location.getBlockX() + "," + location.getBlockY() + "," + location.getBlockZ();
        if (this.isImplicit) {
            coordinates += inheritedSuffix;
        }

        String hVelocity;
        if (this.horizontalVelocity > 0) {
            hVelocity = String.valueOf(this.horizontalVelocity);
        } else {
            hVelocity = getHorizontalVelocity() + inheritedSuffix;
        }

        String vVelocity;
        if (this.verticalVelocity > 0) {
            vVelocity = String.valueOf(this.verticalVelocity);
        } else {
            vVelocity = getVerticalVelocity() + inheritedSuffix;
        }

        return stringFormatter.replacePlaceholders(LaunchpadMessage.SUCCESS_LAUNCHPAD_INFO,
                List.of("{location}", "{horizontalVelocity}", "{verticalVelocity}", "{fixedDirection}"),
                List.of(coordinates, hVelocity, vVelocity, directionString));
    }

}
